# Copyright:   (c) Daryl Fox 2013
# Licence:     CC-BY
#              http://creativecommons.org/licenses/by/3.0/
#-------------------------------------------------------------------------------

import pygame

import config

import game
import game_sounds
import game_menu

__version__ = 0.70

def main():
    pygame.mixer.pre_init(22050, -16, 2, 1024)
    pygame.init()

    #######################################################################
    ## Setup the game screen
    #######################################################################
    winstyle = 0
    bestdepth = pygame.display.mode_ok(config.WINDOW_SIZE, winstyle, 32)
    screen = pygame.display.set_mode(config.WINDOW_SIZE, winstyle, bestdepth)

    #######################################################################
    ## Load assets
    #######################################################################
    effects = game_sounds.load_sounds()

    # This next line forces the focus back to the game at the start when you
    # have mouse focus (Linux) and the mouse is not near the game window at
    # startup.
    pygame.mouse.set_pos((1,1))

    speed = config.PLAYER_SPEED

    effects['theme'].play(loops=-1)
    effects['theme'].set_volume(0.2)
    while True:
        start, speed = game_menu.start_menu(screen, effects, speed)
        if not start:
            break
        level = 1
        playing = True
        while playing:
            playing, stats = game.main(screen, effects, speed, level)
            effects['theme'].fadeout(500)
            if playing:
                effects['win'].play()
                title = 'Level %d: Victory' % stats['Level']
                game_menu.level_results(screen, effects, title, stats)
                level += 1
                effects['win'].fadeout(500)
            else:
                effects['loose'].play()
                title = 'Level %d: Defeat' % stats['Level']
                game_menu.level_results(screen, effects, title, stats)
                effects['loose'].fadeout(500)
            effects['theme'].play(loops=-1)
            effects['theme'].set_volume(0.2)

    pygame.quit()

if __name__ == '__main__':
    main()
