#-------------------------------------------------------------------------------
# Name:        game_blocks
# Purpose:
#
# Author:      Daryl
#
# Created:     17/04/2013
# Copyright:   (c) Daryl 2013
# Licence:     <your licence>
#-------------------------------------------------------------------------------
import pygame

import random

import util

class Blockage(pygame.sprite.Sprite):
    def __init__(self, pos, images, rate=2):
        pygame.sprite.Sprite.__init__(self)
        self.pos = pos
        self._images = images
        self._masks = [pygame.mask.from_surface(i) for i in self._images]
        self.rate = rate

        self.animated = len(self._images) > 1
        self.image_ix = 0
        self.frames = 0.0

        self.rect = self.image.get_rect()

    @property
    def image(self):
        return self._images[self.image_ix]
    @property
    def mask(self):
        return self._mask[self.image_ix]

    def update(self, dt_seconds, dx):
        if self.animated:
            self.frames += dt_seconds
            if self.frames >= self.rate:
                self.image_ix = (self.image_ix + 1) % len(self._images)
                self.frames = 0

        self.pos = (self.pos[0] + dx, self.pos[1])
        self.rect.center = self.pos

        if self.rect.right <= 0:
            self.kill()

#######################################################################
## This class is only used for testing.
#######################################################################
class SquareBlockage(Blockage):
    def __init__(self, pos, color):
        img = pygame.Surface((25, 25))
        img.fill(color)
        super(SquareBlockage, self).__init__(pos, [img])

#######################################################################
## Functions used for testing.
#######################################################################
def generate_generic_column(x, height, y_separation):
    obstructions = []

    colors = [(255, 0, 0), (0, 0, 255), (255, 0, 255), (0, 255, 255)]
    min_sep = int(y_separation * 1.2)
    max_sep = y_separation * 3
    max_x = 0

    y = random.randrange(0, max_sep)
    while y < height:
        c = random.choice(colors)
        x_bump = random.randrange(0, 25)
        block = SquareBlockage((x + x_bump, y), c)
        obstructions.append(block)
        max_x = max(max_x, block.rect.right)
        y += random.randrange(min_sep, max_sep)

    return obstructions, max_x

def generate_generic_field(size, x_start, separation):
    obstructions = []
    min_sep = int(separation[0] * 2)
    max_sep = int(separation[1] * 4)

    x = x_start
    while x < size[0]:
        col, max_x = generate_generic_column(x, size[1], separation[1])
        obstructions.extend(col)
        x += random.randrange(min_sep, max_sep)
    return obstructions, max_x

#######################################################################
## Functions to build actual blockages
#######################################################################
def generate_column(x, height, y_separation, sprite_sizes):
    obstructions = []

    sprites = sprite_sizes.keys()
    min_sep = int(y_separation * 1.33)
    max_sep = int(y_separation * 2.25)
    max_x = 0
    pickups = []

    y = random.randrange(0, max_sep)
    while y < height:
        sprite = random.choice(sprites)
        size = sprite_sizes[sprite]
        x_bump = random.randrange(0, 25)
        block = sprite((x + x_bump, y))
        block.update(0, 0)  # Flicker Reduction
        obstructions.append(block)
        max_x = max(max_x, x_bump)
        y_step = random.randrange(min_sep, max_sep) + block.size[1]
        if random.randint(1, 100) <= 10:
            pickups.append((x, y, y+y_step))
        y += y_step
    return obstructions, max_x, pickups

def generate_field(size, x_start, separation, sprite_sizes):
    obstructions = []
    min_sep = int(separation[0] * 1.75)
    max_sep = int(separation[1] * 3.75)
    pickups = []

    x = x_start
    while x < size[0]:
        col, max_x, p = generate_column(x, size[1], separation[1], sprite_sizes)
        obstructions.extend(col)
        x += random.randrange(min_sep, max_sep) + max_x
        pickups.extend(p)
    return obstructions, max_x, pickups

# Okay, I had a really cool metaclass way of doing this but... it didn't
# work.  Then I came up with a great sprite closure idea that also failed.
# Finally, I realised that (a) I only have a hand full of sprites and (b) it's
# nearly 1:00 AM.  I'm doing it be hand.

class Image_Blockage(pygame.sprite.Sprite):
    _images = []
    _size = (0, 0)
    def __init__(self, pos, rate=2):
        pygame.sprite.Sprite.__init__(self)
        self.pos = pos
        self._masks = [pygame.mask.from_surface(i) for i in self._images]
        self.rate = rate

        self.animated = len(self._images) > 1
        self.image_ix = 0
        self.frames = 0.0

        self.rect = self.image.get_rect()

    @property
    def image(self):
        return self._images[self.image_ix]
    @property
    def mask(self):
        return self._mask[self.image_ix]
    @property
    def size(self):
        return self._size

    def update(self, dt_seconds, dx):

        if self.animated:
            self.frames += dt_seconds
            if self.frames >= self.rate:
                self.image_ix = (self.image_ix + 1) % len(self._images)
                self.frames = 0

        self.pos = (self.pos[0] + dx, self.pos[1])
        self.rect.center = self.pos

        if self.rect.right <= 0:
            self.kill()

##def load_image(filename):
##    image_raw = pygame.image.load(filename)
##    image_raw = image_raw.convert_alpha()
##    size = image_raw.get_size()
##    alpha = image_raw.get_at((0,0))
##
##    image = pygame.Surface(size)  # Do not use SRCALPHA with set_colorkey
##    image.set_colorkey(alpha)
##    image.blit(image_raw, (0,0))
##
##    return [image], image.get_size()

def load_obstructions():
    class Bush1(Image_Blockage):
        filename = 'assets/obstructions/bush1.png'
        _images, _size = util.load_image_list(filename)

    class Bush2(Image_Blockage):
        filename = 'assets/obstructions/bush2.png'
        _images, _size = util.load_image_list(filename)

    class Bush3(Image_Blockage):
        filename = 'assets/obstructions/bush3.png'
        _images, _size = util.load_image_list(filename)

    class Post1(Image_Blockage):
        filename = 'assets/obstructions/post1.png'
        _images, _size = util.load_image_list(filename)

    class Post2(Image_Blockage):
        filename = 'assets/obstructions/post2.png'
        _images, _size = util.load_image_list(filename)

    class Post3(Image_Blockage):
        filename = 'assets/obstructions/post3.png'
        _images, _size = util.load_image_list(filename)

    class Post4(Image_Blockage):
        filename = 'assets/obstructions/post4.png'
        _images, _size = util.load_image_list(filename)

    class Post5(Image_Blockage):
        filename = 'assets/obstructions/post5.png'
        _images, _size = util.load_image_list(filename)

    class Rock1(Image_Blockage):
        filename = 'assets/obstructions/rock1.png'
        _images, _size = util.load_image_list(filename)

    class Rock2(Image_Blockage):
        filename = 'assets/obstructions/rock2.png'
        _images, _size = util.load_image_list(filename)

    class Totem(Image_Blockage):
        filename = 'assets/obstructions/totem.png'
        _images, _size = util.load_image_list(filename)

    class Flare(Image_Blockage):
        filename = 'assets/flare.png'
        _images, _size = util.basic_image_list(filename)

    sprites = (Bush1, Bush2, Bush3,
               Post1, Post3, Post5,
               Post2, Post4,
               Rock1, Rock2, Totem)
    sizes = dict((obj, obj._size) for obj in sprites)
    return sprites, sizes, Flare

def main():
    from pprint import pprint
    MAX_FPS = 60
    WINDOW_SIZE = 640, 480

    pygame.init()

    # set the display mode
    winstyle = 0
    bestdepth = pygame.display.mode_ok(WINDOW_SIZE, winstyle, 32)
    screen = pygame.display.set_mode(WINDOW_SIZE, winstyle, bestdepth)
    clock = pygame.time.Clock()

    sprites, heights = load_obstructions()
    pprint(heights)
    ix = 0
    sprite = sprites[ix](screen.get_rect().center)

    running = True
    while running:
        dt = clock.tick(MAX_FPS)
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    running = False
                elif event.key == pygame.K_SPACE:
                    ix = (ix + 1) % len(sprites)
                    sprite = sprites[ix](screen.get_rect().center)
        if not (dt and running):
            continue

        sprite.update(dt / 1000.0, 0)

        screen.fill((32,32,32))
        screen.blit(sprite.image, sprite.rect)

        fps = clock.get_fps()
        pygame.display.set_caption('Goo -- %.01f FPS' % fps)
        pygame.display.flip()

if __name__ == '__main__':
    main()
