import pygame

import math
import sys
import os

def save_screenshot(surface):
    if not os.path.exists('./shots'):
        try:
            os.mkdir('./shots')
        except:
            return

    ix = 0
    while ix < 999:
        filename = os.path.join('./shots', 'shot_%03d.png' % ix)
        if not os.path.exists(filename):
            break
        ix += 1
    else:
        return

    pygame.image.save(surface, filename)

def find_font_by_height(target):
    pygame.font.init()
    name = pygame.font.get_default_font()

    size = 120
    while size >= 6:
        font = pygame.font.Font(name, size)
        if font.get_height() <= target:
            return font
        size -= 1
    raise FontError("Cannot find font of size %r" % target)

def find_font_by_width(target, text):
    pygame.font.init()
    name = pygame.font.get_default_font()

    size = 120
    while size >= 6:
        font = pygame.font.Font(name, size)
        if font.size(text)[0] <= target:
            return font
        size -= 1
    raise FontError("Cannot find font of size %r" % target)

def load_image(filename):
    image_raw = pygame.image.load(filename)
    image_raw = image_raw.convert_alpha()
    size = image_raw.get_size()
    alpha = image_raw.get_at((0,0))

    image = pygame.Surface(size)  # Do not use SRCALPHA with set_colorkey
    image.set_colorkey(alpha)
    image.blit(image_raw, (0,0))

    return image, image.get_size()

def load_image_list(filename):
    image, size = load_image(filename)
    return [image], size

def basic_image_list(filename):
    image = pygame.image.load(filename)
    image = image.convert_alpha()
    return [image], image.get_size()

def calc_bomblet_positions(radius, pos):
    rr = radius * 2
    sin60 = math.sin(math.radians(60))  # 0.866
    cos60 = math.cos(math.radians(60))  # 0.5

    b0  = (-rr, 0)
    b1  = (-rr * cos60,  rr * sin60)
    b1n = (b1[0], -b1[1])
    #b2  = (rr * sin60 + b1[0], rr * cos60 + b1[1])
    b2  = (rr + b1[0], b1[1])
    b2n = (b2[0], -b2[1])

    bomblets = (b0, b1, b1n, b2, b2n)
    return [[int(pos[0] + x), int(pos[1] + y)] for x, y in (b0, b1, b1n, b2, b2n)]


